@@ -17,52 +17,75 @@ base_path = os.path.dirname(os.path.abspath(__file__))
 
 
 class SwaggerUIPlugin(BasePlugin):
-    """ Create Swagger UI with swagger-ui tag """
+    """Create Swagger UI with swagger-ui tag"""
 
     config_scheme = (
         ("background", config_options.Type(str, default="")),
         # Display
-        ("docExpansion",
-         config_options.Choice(("list", "full", "none"), default="list")),
+        (
+            "docExpansion",
+            config_options.Choice(("list", "full", "none"), default="list"),
+        ),
         ("filter", config_options.Type(object, default=False)),
-        ("syntaxHighlightTheme",
-         config_options.Choice(("agate", "arta", "monokai", "nord", "obsidian",
-                                "tomorrow-night"),
-                               default="agate")),
+        (
+            "syntaxHighlightTheme",
+            config_options.Choice(
+                ("agate", "arta", "monokai", "nord", "obsidian", "tomorrow-night"),
+                default="agate",
+            ),
+        ),
         ("tryItOutEnabled", config_options.Type(bool, default=False)),
         # Network
         ("oauth2RedirectUrl", config_options.Type(str, default=None)),
-        ("supportedSubmitMethods",
-         config_options.Type(list,
-                             default=[
-                                 "get", "put", "post", "delete", "options",
-                                 "head", "patch", "trace"
-                             ])),
-        ("validatorUrl", config_options.Type(str, default=None)),
+        (
+            "supportedSubmitMethods",
+            config_options.Type(
+                list,
+                default=[
+                    "get",
+                    "put",
+                    "post",
+                    "delete",
+                    "options",
+                    "head",
+                    "patch",
+                    "trace",
+                ],
+            ),
+        ),
+        ("validatorUrl", config_options.Type(str, default="none")),
+        ("extra_css", config_options.Type(list, default=[])),
     )
 
     def on_pre_page(self, page, config, files, **kwargs):
-        """ Add files for validate swagger-ui tag src """
+        """Add files for validate swagger-ui tag src"""
 
         self.files = files
         return page
 
     def path_to_url(self, page_file, url):
-        """ Validate swagger-ui tag src and parse url """
+        """Validate swagger-ui tag src and parse url"""
 
         scheme, netloc, path, query, fragment = urlsplit(url)
 
-        if (scheme or netloc or not path or url.startswith('/')
-                or url.startswith('\\') or AMP_SUBSTITUTE in url
-                or '.' not in os.path.split(path)[-1]):
+        if (
+            scheme
+            or netloc
+            or not path
+            or url.startswith("/")
+            or url.startswith("\\")
+            or AMP_SUBSTITUTE in url
+            or "." not in os.path.split(path)[-1]
+        ):
             # Ignore URLs unless they are a relative link to a source file.
             # AMP_SUBSTITUTE is used internally by Markdown only for email.
             # No '.' in the last part of a path indicates path does not point to a file.
             return url
 
         # Determine the filepath of the target.
-        target_path = os.path.join(os.path.dirname(page_file.src_path),
-                                   urlunquote(path))
+        target_path = os.path.join(
+            os.path.dirname(page_file.src_path), urlunquote(path)
+        )
         target_path = os.path.normpath(target_path).lstrip(os.sep)
 
         # Validate that the target exists in files collection.
@@ -79,26 +102,11 @@ class SwaggerUIPlugin(BasePlugin):
         return urlunsplit(components)
 
     def on_post_page(self, output, page, config, **kwargs):
-        """ Replace swagger-ui tag with iframe
-            Add javascript code to update iframe height
-            Create a html with Swagger UI for iframe
+        """Replace swagger-ui tag with iframe
+        Add javascript code to update iframe height
+        Create a html with Swagger UI for iframe
         """
 
-        css_dir = utils.get_relative_url(
-            utils.normalize_url("assets/stylesheets/"), page.url)
-        js_dir = utils.get_relative_url(
-            utils.normalize_url("assets/javascripts/"), page.url)
-        default_oauth2_redirect_file = utils.get_relative_url(
-            utils.normalize_url("assets/swagger-ui/oauth2-redirect.html"),
-            page.url)
-        env = Environment(
-            loader=FileSystemLoader(os.path.join(base_path, "swagger-ui")))
-        template = env.get_template('swagger.html')
-
-        page_dir = os.path.dirname(os.path.join(config['site_dir'], urlunquote(page.url)))
-        if not os.path.exists(page_dir):
-            os.makedirs(page_dir)
-
         soup = BeautifulSoup(output, "html.parser")
         swagger_ui_list = soup.find_all("swagger-ui")
         iframe_id_list = []
@@ -107,6 +115,30 @@ class SwaggerUIPlugin(BasePlugin):
         if len(swagger_ui_list) == 0:
             return str(soup)
 
+        css_dir = utils.get_relative_url(
+            utils.normalize_url("assets/stylesheets/"), page.url
+        )
+        js_dir = utils.get_relative_url(
+            utils.normalize_url("assets/javascripts/"), page.url
+        )
+        default_oauth2_redirect_file = utils.get_relative_url(
+            utils.normalize_url("assets/swagger-ui/oauth2-redirect.html"), page.url
+        )
+        env = Environment(
+            loader=FileSystemLoader(os.path.join(base_path, "swagger-ui"))
+        )
+        template = env.get_template("swagger.html")
+        extra_css_files = list(map(
+            lambda f: utils.get_relative_url(utils.normalize_url(f), page.url),
+            self.config["extra_css"],
+        ))
+
+        page_dir = os.path.dirname(
+            os.path.join(config["site_dir"], urlunquote(page.url))
+        )
+        if not os.path.exists(page_dir):
+            os.makedirs(page_dir)
+
         for swagger_ui_ele in swagger_ui_list:
             if swagger_ui_ele.has_attr("grouped"):
                 grouped_list.append(swagger_ui_ele)
@@ -117,25 +149,28 @@ class SwaggerUIPlugin(BasePlugin):
             iframe_id_list.append(cur_id)
             cur_options = self.process_options(config, swagger_ui_ele)
             cur_oath2_prop = self.process_oath2_prop(swagger_ui_ele)
-            oauth2_redirect_url = cur_options.pop('oauth2RedirectUrl', '')
+            oauth2_redirect_url = cur_options.pop("oauth2RedirectUrl", "")
             if not oauth2_redirect_url:
                 oauth2_redirect_url = default_oauth2_redirect_file
 
-            openapi_spec_url = self.path_to_url(page.file,
-                                                swagger_ui_ele.get('src', ""))
+            openapi_spec_url = self.path_to_url(
+                page.file, swagger_ui_ele.get("src", "")
+            )
             output_from_parsed_template = template.render(
                 css_dir=css_dir,
+                extra_css_files=extra_css_files,
                 js_dir=js_dir,
-                background=self.config['background'],
+                background=self.config["background"],
                 id=cur_id,
                 openapi_spec_url=openapi_spec_url,
                 oauth2_redirect_url=oauth2_redirect_url,
+                validatorUrl=self.config["validatorUrl"],
                 options_str=json.dumps(cur_options, indent=4)[1:-1],
-                oath2_prop_str=json.dumps(cur_oath2_prop))
-            with open(os.path.join(page_dir, iframe_filename), 'w') as f:
+                oath2_prop_str=json.dumps(cur_oath2_prop),
+            )
+            with open(os.path.join(page_dir, iframe_filename), "w") as f:
                 f.write(output_from_parsed_template)
-            self.replace_with_iframe(soup, swagger_ui_ele, cur_id,
-                                     iframe_filename)
+            self.replace_with_iframe(soup, swagger_ui_ele, cur_id, iframe_filename)
 
         if grouped_list:
             cur_id = str(uuid.uuid4())[:8]
@@ -143,32 +178,32 @@ class SwaggerUIPlugin(BasePlugin):
             iframe_id_list.append(cur_id)
             openapi_spec_url = []
             for swagger_ui_ele in grouped_list:
-                cur_url = self.path_to_url(page.file,
-                                           swagger_ui_ele.get('src', ""))
-                cur_name = swagger_ui_ele.get('name',
-                                              swagger_ui_ele.get('src', ""))
+                cur_url = self.path_to_url(page.file, swagger_ui_ele.get("src", ""))
+                cur_name = swagger_ui_ele.get("name", swagger_ui_ele.get("src", ""))
                 openapi_spec_url.append({"url": cur_url, "name": cur_name})
 
             # only use options from first grouped swagger ui tag
             cur_options = self.process_options(config, grouped_list[0])
             cur_oath2_prop = self.process_oath2_prop(grouped_list[0])
-            oauth2_redirect_url = cur_options.pop('oauth2RedirectUrl', '')
+            oauth2_redirect_url = cur_options.pop("oauth2RedirectUrl", "")
             if not oauth2_redirect_url:
                 oauth2_redirect_url = default_oauth2_redirect_file
 
             output_from_parsed_template = template.render(
                 css_dir=css_dir,
+                extra_css_files=extra_css_files,
                 js_dir=js_dir,
-                background=self.config['background'],
+                background=self.config["background"],
                 id=cur_id,
                 openapi_spec_url=openapi_spec_url,
                 oauth2_redirect_url=oauth2_redirect_url,
+                validatorUrl=self.config["validatorUrl"],
                 options_str=json.dumps(cur_options, indent=4)[1:-1],
-                oath2_prop_str=json.dumps(cur_oath2_prop))
-            with open(os.path.join(page_dir, iframe_filename), 'w') as f:
+                oath2_prop_str=json.dumps(cur_oath2_prop),
+            )
+            with open(os.path.join(page_dir, iframe_filename), "w") as f:
                 f.write(output_from_parsed_template)
-            self.replace_with_iframe(soup, grouped_list[0], cur_id,
-                                     iframe_filename)
+            self.replace_with_iframe(soup, grouped_list[0], cur_id, iframe_filename)
             # only keep first grouped swagger ui tag
             for rest_swagger_ui_ele in grouped_list[1:]:
                 rest_swagger_ui_ele.extract()
@@ -242,23 +277,22 @@ class SwaggerUIPlugin(BasePlugin):
 
         return str(soup)
 
-    def replace_with_iframe(self, soup, swagger_ui_ele, cur_id,
-                            iframe_filename):
-        """ Replace swagger-ui tag with iframe """
+    def replace_with_iframe(self, soup, swagger_ui_ele, cur_id, iframe_filename):
+        """Replace swagger-ui tag with iframe"""
         iframe = soup.new_tag("iframe")
-        iframe['id'] = cur_id
-        iframe['src'] = iframe_filename
-        iframe['frameborder'] = "0"
-        iframe['style'] = "overflow:hidden;width:100%;"
-        iframe['width'] = "100%"
-        iframe['class'] = "swagger-ui-iframe"
+        iframe["id"] = cur_id
+        iframe["src"] = iframe_filename
+        iframe["frameborder"] = "0"
+        iframe["style"] = "overflow:hidden;width:100%;"
+        iframe["width"] = "100%"
+        iframe["class"] = "swagger-ui-iframe"
         swagger_ui_ele.replaceWith(iframe)
 
     def process_options(self, config, swagger_ui_ele):
-        """ Retrieve Swagger UI options from attribute and use config options as default """
+        """Retrieve Swagger UI options from attribute and use config options as default"""
+        skip_option_keys = ["background", "custom_css_files"]
         global_options = {
-            k: v
-            for k, v in dict(self.config).items() if k != 'background'
+            k: v for k, v in dict(self.config).items() if k not in skip_option_keys
         }
         options_keys = global_options.keys()
         cur_options = {}
@@ -274,9 +308,11 @@ class SwaggerUIPlugin(BasePlugin):
                             )
                         cur_options[k] = val
                     except Exception as e:
-                        logging.error(e)
-                        logging.error(
-                            "Ignore supportedSubmitMethods attribute setting.")
+                        logging.warning(e)
+                        logging.warning(
+                            "Ignore supportedSubmitMethods attribute setting."
+                        )
+                        cur_options[k] = None
                 else:
                     cur_options[k] = val
             else:
@@ -285,11 +321,12 @@ class SwaggerUIPlugin(BasePlugin):
                 cur_options.pop(k)
         if "syntaxHighlightTheme" in cur_options:
             cur_options["syntaxHighlight.theme"] = cur_options.pop(
-                "syntaxHighlightTheme")
+                "syntaxHighlightTheme"
+            )
         return cur_options
 
     def process_oath2_prop(self, swagger_ui_ele):
-        """ Retrieve Swagger UI OAuth 2.0 configuration from tag attributes """
+        """Retrieve Swagger UI OAuth 2.0 configuration from tag attributes"""
         oath_prop_keys = [
             "clientId",
             "clientSecret",
@@ -313,13 +350,13 @@ class SwaggerUIPlugin(BasePlugin):
                             )
                         cur_prop[k] = val
                     except Exception as e:
-                        logging.error(e)
-                        logging.error(
+                        logging.warning(e)
+                        logging.warning(
                             "Ignore additionalQueryStringParams attribute setting."
                         )
                 elif k in [
-                        "useBasicAuthenticationWithAccessCodeGrant",
-                        "usePkceWithAuthorizationCodeGrant"
+                    "useBasicAuthenticationWithAccessCodeGrant",
+                    "usePkceWithAuthorizationCodeGrant",
                 ]:
                     cur_prop[k] = val.lower().strip() == "true"
                 else:
@@ -327,26 +364,31 @@ class SwaggerUIPlugin(BasePlugin):
         return cur_prop
 
     def on_post_build(self, config, **kwargs):
-        """ Copy Swagger UI css and js files to assets directory """
+        """Copy Swagger UI css and js files to assets directory"""
 
         output_base_path = os.path.join(config["site_dir"], "assets")
         css_path = os.path.join(output_base_path, "stylesheets")
         for file_name in os.listdir(
-                os.path.join(base_path, "swagger-ui", "stylesheets")):
+            os.path.join(base_path, "swagger-ui", "stylesheets")
+        ):
             utils.copy_file(
-                os.path.join(base_path, "swagger-ui", "stylesheets",
-                             file_name), os.path.join(css_path, file_name))
+                os.path.join(base_path, "swagger-ui", "stylesheets", file_name),
+                os.path.join(css_path, file_name),
+            )
 
         js_path = os.path.join(output_base_path, "javascripts")
         for file_name in os.listdir(
-                os.path.join(base_path, "swagger-ui", "javascripts")):
+            os.path.join(base_path, "swagger-ui", "javascripts")
+        ):
             utils.copy_file(
-                os.path.join(base_path, "swagger-ui", "javascripts",
-                             file_name), os.path.join(js_path, file_name))
+                os.path.join(base_path, "swagger-ui", "javascripts", file_name),
+                os.path.join(js_path, file_name),
+            )
 
         swagger_ui_path = os.path.join(output_base_path, "swagger-ui")
         if not os.path.exists(swagger_ui_path):
             os.mkdir(swagger_ui_path)
         utils.copy_file(
             os.path.join(base_path, "swagger-ui", "oauth2-redirect.html"),
-            os.path.join(swagger_ui_path, "oauth2-redirect.html"))
+            os.path.join(swagger_ui_path, "oauth2-redirect.html"),
+        )
